import setuptools
version = "1.4.1"

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="symlib",
    version=version,
    author="Philip Mansfield",
    author_email="mansfield.astro@gmail.com",
    description="A library for working with data from the Symphony and MW-est zoom-in suites.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["numpy", "scipy", "matplotlib", "colossus", "requests"],
    url="https://github.com/phil-mansfield/symlib",
    keywords=["python"],
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.6"
)
#print(setuptools.find_packages(where="src"))



import numpy as np
import scipy.integrate as integrate
import scipy.interpolate as interpolate
import numpy.random as random
import scipy.stats as stats

class PDF(object):
    def __init__(self, pdf, low, high):
        self.pdf = pdf

        x = np.linspace(low, high, 150)
        y = np.zeros(len(x))

        for i in range(1, len(y)):
            y[i] = integrate.quad(pdf, low, x[i])[0]
        y /= y[-1]
        
        self.low, self.high = low, high
        self.cdf = interpolate.interp1d(x, y, "linear")
        ok = np.ones(len(x))
        ok[1:] = y[1:] != y[:-1]
        self.inverse_cdf = interpolate.interp1d(y, x, "linear")

    def mean(self):
        return (integrate.quad(lambda x: self.pdf(x)*x, self.low, self.high)[0]/
                integrate.quad(lambda x: self.pdf(x), self.low, self.high)[0])

    def sigma(self):
        x = self.mean()
        x2 = (integrate.quad(lambda x: self.pdf(x)*x*x, self.low, self.high)[0]/
              integrate.quad(lambda x:self.pdf(x), self.low, self.high)[0])
        return np.sqrt(x2 - x*x)

    def sample(self, n):
        y = random.random(n)
        return self.inverse_cdf(y)

def gaussian_coupala_sample(x, y_cdf, rho):
    n = len(x)

    r = random.multivariate_normal([0, 0], [[1, rho], [rho, 1]], n)
    xr, yr = r[:,0], r[:,1]
    i_xr = stats.rankdata(xr, "ordinal")-1
    i_yr = stats.rankdata(yr, "ordinal")-1

    x_order = np.argsort(x)
    x_idx = np.arange(n, dtype=int)
    x_orig_idx = x_idx[x_order]

    y_low, y_high = y_cdf[1,0], y_cdf[1,-1]
    y_range = y_high - y_low
    inverse_y_cdf = interpolate.interp1d(y_cdf[1,:], y_cdf[0,:])
    y_rand = inverse_y_cdf(random.random(len(x))*y_range + y_low)
    y_sort = np.sort(y_rand)
    y = y_sort[i_yr]

    xr_order = np.argsort(xr)

    out = np.zeros(n)
    out[x_orig_idx] = y[xr_order]

    return out



# Data types
from .lib import SUBHALO_DTYPE, HISTORY_DTYPE, BRANCH_DTYPE, CORE_DTYPE, UM_DTYPE, PARTICLE_DTYPE, GALAXY_DTYPE, GALAXY_HISTORY_DTYPE, STAR_DTYPE
# I/O function
from .lib import read_subhalos, read_cores, read_branches, read_tree, read_particles, read_um, read_galaxies, ParticleInfo
from .lib import Particles
# Utilities
from .lib import simulation_parameters, parameter_table, scale_factors, pristine_merger_indices, merger_stats, propagate_parent_indices, colossus_parameters, suite_names, merger_lookup_table, find_merger_branch, find_all_merger_branches, is_real_confirmed, read_symfind, read_rockstar, pre_infall_host, transform_smooth_particles
# TODO: better names for pristine_merger_indices and propagate_parent_indices
from .match import match_subhalos, plot_matched_subhalos

# Abstract models
from .star_tagging import ProfileShapeModel, RHalfModel, MStarModel, AbstractRanking, FeHMeanModel, FeHMDFModel, FeHProfileModel, MetalCorrelationModel, SFHModel
# ProfileShape models
from .star_tagging import PlummerProfile
# R_half models
from .star_tagging import Nadler2020RHalf, Jiang2019RHalf, FixedRHalf
# Metallicity models
from .star_tagging import Kirby2013Metallicity, Kirby2013MDF, FlatFeHProfile
# M_star models
from .star_tagging import UniverseMachineMStarFit, UniverseMachineMStar
# SFH models
from.star_tagging import UniverseMachineSFH, DarkMatterSFH
# Metal correlation models
from .star_tagging import GaussianCoupalaCorrelation
# Ranking schemes
from .star_tagging import RadialEnergyRanking
# Galaxy-halo model components
from .star_tagging import StellarMassModel, ProfileModel, MetalModel
# Galaxy-halo models
from .star_tagging import GalaxyHaloModel
# Utility functions
from .star_tagging import clean_particles, tag_stars, look_back_orbital_time, profile_info, rmax_vmax, retag_stars

# Default models:
from .star_tagging import DWARF_GALAXY_HALO_MODEL, DWARF_GALAXY_HALO_MODEL_NO_UM

# Unit conversion
from .util import set_units_halos, set_units_x, set_units_v, set_units_parameters, set_units_histories
# Halo names
from .util import  get_host_directory, n_hosts

# These are functions that are only really useful for the the tutorial
from .util import plot_circle

# File management
from .file_management import pack_files, download_packed_files, download_files, unpack_files
